import logging
import sys

from loguru import logger
from split_settings.tools import include, optional

include(
    "../common_components/base.py",
    "components/worker_apps.py",
    "../common_components/channels.py",
    "components/discord.py",
    "../common_components/django_extensions.py",
    optional("components/local_settings.py"),
)


class InterceptHandler(logging.Handler):
    def emit(self, record):
        # Get corresponding Loguru level if it exists
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())


logging.basicConfig(handlers=[InterceptHandler()], level=0)


logger.remove()
logger_format = (
    "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | "
    "<level>{level: <8}</level> | "
    "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> | "
    "<red>Extra: {extra}</red> - <level>{message}</level>"
)

logger.add(sys.stdout,  colorize=True, format=logger_format, level="INFO")



from split_settings.tools import include, optional

include(
    "../common_components/base.py",
    "../common_components/channels.py",
    "../common_components/django_extensions.py",
    optional("components/local_settings.py"),
)



from typing import ClassVar

from discord.ext.commands import Bot

from django_discord.py.exceptions import BotNotReadyYet


class DjangoDiscordPlugin:
    command_definitions = []

    bot_proxy: ClassVar = None

    @property
    def bot(self) -> Bot:
        the_bot = type(self).bot_proxy
        if not the_bot:
            raise BotNotReadyYet("The bot isn't ready yet! Have you started the bot?")
        return the_bot



from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer
from django.apps import AppConfig
from django.conf import settings
from loguru import logger


class DjangoDiscordPyBotAutoStartConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_discord.py.bot"

    def ready(self):
        channel_layer = get_channel_layer()
        logger.info("Sending start signal")

        logging_config = {
            config_key: getattr(settings, config_setting)
            for config_key, config_setting
            in [
                ("log_handler", 'DISCORD_BOT_LOG_HANDLER', ),
                ("log_formatter", 'DISCORD_BOT_LOG_FORMATTER', ),
                ("log_level", 'DISCORD_BOT_LOG_LEVEL', ),
                ("root_logger", 'DISCORD_BOT_ROOT_LOGGER', ),
            ]
            if hasattr(settings, config_setting)
        }
        async_to_sync(channel_layer.send)(
            "discord_bot",
            {
                "type": "start.discord.bot",
                'plugins': settings.DISCORD_BOT_PLUGINS,
                "bot_path": settings.DISCORD_BOT_PATH,
                "reconnect": settings.DISCORD_BOT_RECONNECT,
                **logging_config,
            }
        )
        logger.info("Start signal sent")



import os

DISCORD_PY_BOT_TOKEN = os.getenv('DISCORD_BOT_TOKEN', 'SET_THIS')
DISCORD_PY_COMMAND_PREFEX = "?"
DISCORD_BOT_PATH = 'django_discord.py.bot.basic_bot:bot'
DISCORD_BOT_RECONNECT = True
DISCORD_BOT_LOG_LEVEL = 'DEBUG'
DISCORD_BOT_PLUGINS = [
    'django_discord.py.example_bot.bot_plugins',
]



from typing import Any

import discord
from discord.ext import commands
from discord.ext.commands import Context
from django.conf import settings
from loguru import logger

from django_discord.py.bot.signals import bot_error, bot_ready, command_error

description = """
A bot for friendly betting on league of legends
"""

intents = discord.Intents.default()
intents.message_content = True

bot = commands.Bot(command_prefix=settings.DISCORD_PY_COMMAND_PREFEX, description=description, intents=intents)

log = logger


@bot.event
async def on_command_error(ctx: Context, error):
    log.bind(**{'ctx': ctx, 'error': error}).error("Discord bot had an error!")
    command_error.send_robust(sender=bot.__class__, ctx=ctx, error=error)
    if not command_error.receivers:
        await ctx.send("Something went wrong! Add a listener to command_error to replace this message!")


@bot.event
async def on_error(event_method: str, /, *args: Any, **kwargs: Any):
    log.bind(**{'event_method': event_method, 'args': args, 'kwargs': kwargs}).error("Discord bot had an error!")
    bot_error.send_robust(sender=bot.__class__, event_method=event_method, args=args, kwargs=kwargs)


@bot.event
async def on_ready():
    logger.info(f"Logged in as {bot.user} (ID: {bot.user.id})")
    logger.info("------")
    bot_ready.send_robust(sender=bot.__class__, bot=bot)



ASGI_APPLICATION = "django_discord.py.configuration.asgi.application"

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels_redis.core.RedisChannelLayer",
        "CONFIG": {
            "hosts": [("127.0.0.1", 6379)],
        },
    },
}



import asyncio
import logging
from importlib import import_module
from typing import Optional

from channels.consumer import AsyncConsumer
from daphne.utils import import_by_path
from discord import utils
from discord.ext.commands import Bot
from discord.utils import MISSING
from django.conf import settings
from loguru import logger

from django_discord.py.plugins.datatypes import DjangoDiscordPlugin


class BaseDiscordConsumer(AsyncConsumer):
    bot_task: asyncio.Task = None
    bot: Bot = None

    async def start_discord_bot(self, event):
        logger.info("Received start signal")
        if await self.is_bot_running():
            logger.info("Bot is already running, doing nothing")
            return

        logger.info("Unpacking signal for bot parameters")
        bot_path = event['bot_path']
        reconnect: bool = event['reconnect']

        log_handler: Optional[logging.Handler] = event.get('log_handler', MISSING)
        log_formatter: logging.Formatter = event.get('log_formatter', MISSING)
        log_level: int = event.get('log_level', MISSING)
        root_logger: bool = event.get('root_logger', False)

        logger.info(f"Importing '{bot_path}' ...")
        self.bot: Bot = import_by_path(bot_path)

        logger.info("Grabbing bot auth token from settings")
        bot_token: str = settings.DISCORD_PY_BOT_TOKEN

        async def runner(bot, token, reconnect):
            async with bot:
                await bot.start(token, reconnect=reconnect)

        if log_handler is not None:
            utils.setup_logging(
                handler=log_handler,
                formatter=log_formatter,
                level=log_level,
                root=root_logger,
            )

        logger.info("Loading plugins")
        plugin_paths = event.get('plugins', [])
        DjangoDiscordPlugin.bot_proxy = self.bot
        for plugin_path in plugin_paths:
            module = import_module(plugin_path)
            plugin = getattr(module, 'plugin')
            if not plugin:
                logger.error("Improperly configured plugin module found %s. Where is the `plugin` attribute?", module)

            try:
                await module.setup(self.bot)
            except AttributeError:
                pass
            except TypeError:
                logger.error("%s.setup should be an async function!", module)
                raise

        logger.info("Creating bot loop")
        bot_coro = runner(self.bot, bot_token, reconnect)

        logger.info("Running bot")
        self.bot_task = asyncio.create_task(bot_coro)

    async def is_bot_running(self):
        if not self.bot_task:
            return False

        return not self.bot_task.done() and not self.bot_task.cancelled()

    async def send_message(self, event: dict):
        if not self.is_bot_running():
            raise ValueError("Bot is not running!")
        event.pop('type')
        channel: int = event.pop('channel')
        content: str = event.pop('content', None)

        await self.bot.get_channel(channel).send(
            content=content,
            **event,
        )



import sys

from loguru import logger

from django_discord.py.cli import run_daphne_server

if __name__ == "__main__":
    ip_address = '0.0.0.0'
    port = '8075'
    try:
        ip_address, port = sys.argv[1:]
    except ValueError:
        try:
            ip_address = sys.argv[1]
        except IndexError:
            # Use the default, probably a developer
            pass

    except Exception as e:
        logger.error(f"We did not understand that, sorry! Here was the error: {e}")
        sys.exit(1)

    run_daphne_server(ip_address, port)



import logging
from typing import Optional, Union

import discord
from discord import Interaction, app_commands
from discord.ext.commands import Bot, Context
from django.dispatch import receiver

from django_discord.py.bot.signals import bot_error, bot_ready, command_error
from django_discord.py.plugins.datatypes import DjangoDiscordPlugin

plugin = DjangoDiscordPlugin()

log = logging.getLogger(__name__)


async def setup(bot: Bot):
    """
    This code runs as soon as the bot is available to your plugin, it won't necessarily
    be ready.
    """
    # Do not sync commands! You'll be rate limited.
    ...


@receiver(bot_ready, sender=Bot)
def my_handler(sender, bot: Bot, **kwargs):
    """
    Add your bot ready code here! But do not sync commands! You'll be rate limited.
    """
    ...


@receiver(command_error, sender=Bot)
def on_command_error(sender, bot: Bot, event_method: str, args, kwargs):
    """
    Add your server error handling code here!
    """


@receiver(bot_error, sender=Bot)
def on_error(sender, bot: Bot, ctx: Context, error: Exception, **kwargs, ):
    """
    Add your bot error handling code here!
    """
    await ctx.send(f"Something went wrong! {error}")


@plugin.bot.command(description="Sync commands to servers")
async def sync(ctx: Context):
    await plugin.bot.tree.sync(guild=discord.Object(id=ctx.guild.id))
    await ctx.send(
        "Syncing commands now!"
    )


@plugin.bot.tree.command(guilds=[discord.Object(id=802682445240991744)])
@app_commands.describe(number='Any number', string='Some random string')
async def guild_only(interaction: Interaction, number: int, string: str):
    await interaction.response.send_message(f'Modify {number=} {string=}', ephemeral=True)


@plugin.bot.tree.command()
@app_commands.describe(attachment='The file to upload')
async def global_command(interaction: discord.Interaction, attachment: discord.Attachment):
    await interaction.response.send_message(f'Thanks for uploading {attachment.filename}!', ephemeral=True)


@plugin.bot.command(description="Say 'Hello, world!'")
async def howdy(ctx: Context):
    log.info("Someone said 'Howdy', responding!")
    await ctx.send(
        f"Howdy, channel #{ctx.guild.id}! 🤠"
    )


class Permissions(app_commands.Group):
    """Manage permissions of a member."""

    def get_permissions_embed(self, permissions: discord.Permissions) -> discord.Embed:
        embed = discord.Embed(title='Permissions', colour=discord.Colour.blurple())
        permissions = [
            (name.replace('_', ' ').title(), value)
            for name, value in permissions
        ]

        allowed = [name for name, value in permissions if value]
        denied = [name for name, value in permissions if not value]

        embed.add_field(name='Granted', value='\n'.join(allowed), inline=True)
        embed.add_field(name='Denied', value='\n'.join(denied), inline=True)
        return embed

    @app_commands.command()
    @app_commands.describe(target='The member or role to get permissions of')
    async def get(self, interaction: discord.Interaction, target: Union[discord.Member, discord.Role]):
        """Get permissions for a member or role"""

        if isinstance(target, discord.Member):
            assert target.resolved_permissions is not None
            embed = self.get_permissions_embed(target.resolved_permissions)
            embed.set_author(name=target.display_name, url=target.display_avatar)
        else:
            embed = self.get_permissions_embed(target.permissions)

        await interaction.response.send_message(embed=embed)

    @app_commands.command(name='in')
    @app_commands.describe(channel='The channel to get permissions in')
    @app_commands.describe(member='The member to get permissions of')
    async def _in(
            self,
            interaction: discord.Interaction,
            channel: Union[discord.TextChannel, discord.VoiceChannel],
            member: Optional[discord.Member] = None,
    ):
        """Get permissions for you or another member in a specific channel."""
        embed = self.get_permissions_embed(channel.permissions_for(member or interaction.user))
        await interaction.response.send_message(embed=embed)


# Another way to add commands
# To add the Group to your tree...
plugin.bot.tree.add_command(Permissions(), guild=discord.Object(id=802682445240991744))



from django_discord.py.configuration.common_components.base import INSTALLED_APPS

INSTALLED_APPS = [
    *INSTALLED_APPS,
    "django_discord.py.bot",
    "django_discord.py.plugins",
    "django_discord.py.example_bot",
]



import os

from daphne.cli import CommandLineInterface
from django.core.management import execute_from_command_line


def run_daphne_server(ip_address, ip_port):
    CommandLineInterface().run(
        (
            'django_discord.py.configuration.asgi:application',
            '--bind',
            ip_address,
            '--port',
            ip_port,
        )
    )


def run_asgi_workers(*channels):
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "django_discord.py.configuration.worker_settings")

    if not channels:
        channels = ('discord_bot', )

    execute_from_command_line(('manage.py', 'runworker', *channels))



"""
ASGI config for django_discord.py project.

It exposes the ASGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/4.1/howto/deployment/asgi/
"""

import importlib
import os

from channels.routing import ChannelNameRouter, ProtocolTypeRouter
from django.core.asgi import get_asgi_application

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "django_discord.py.configuration.http_settings")

# Initialize Django ASGI application early to ensure the AppRegistry
# is populated before importing code that may import ORM models.

django_asgi_app = get_asgi_application()

# Import after django is setup so we can count on settings being initialized
consumers_module = importlib.import_module('django_discord.py.bot.consumers')

application = ProtocolTypeRouter(
    {
        "http": django_asgi_app,
        "channel": ChannelNameRouter(
            {
                "discord_bot": consumers_module.BaseDiscordConsumer.as_asgi(),
            }
        ),
    }
)
